﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Web;
using System.Xml;

namespace MatchData
{
    /// <summary>
    /// determine whether the current now match with the condition of the xml string 
    /// </summary>
    internal class MatchWithMatchData
    {
        DateTime startDt;
        DateTime nowDt;
        /// <summary>
        /// initialize data
        /// </summary>
        /// <param name="startDt">start datetime</param>
        /// <param name="nowDt">now datetime</param>
        public void InitData(DateTime startDt, DateTime nowDt)
        {
            this.startDt = startDt;
            this.nowDt = nowDt;
        }

        public bool StartDateTimeFunction(string xml)
        {
            XmlDocument xmldoc = new XmlDocument();
            xmldoc.LoadXml(xml);
            //whether the date and time is in the range of startDateTime
            if (nowDt.Date >= startDt.Date && Math.Abs((startDt.TimeOfDay - nowDt.TimeOfDay).TotalSeconds) < 20)
                return true;
            else
                return false;
        }

        public bool EndDateFunction(string xml)
        {
            XmlDocument xmldoc = new XmlDocument();
            xmldoc.LoadXml(xml);
            DateTime dt = DateTime.Parse(xmldoc["EndDate"].InnerText).ToLocalTime();
            if (nowDt.Date <= dt.Date)
                return true;
            else
                return false;
        }
        //-----------------------------------------------------------------------------
        public bool MonthlyDOWRecurrenceFunction(string xml)
        {
            bool result = true;
            XmlDocument xmldoc = new XmlDocument();
            xmldoc.LoadXml(xml);
            foreach (XmlNode xmlnode in xmldoc.ChildNodes[0].ChildNodes)
            {
                MethodInfo method = this.GetType().GetMethod(xmlnode.Name + "Function", BindingFlags.Instance | BindingFlags.Public);
                object oj = method.Invoke(this, new object[] { xmlnode.OuterXml });
                if (result)
                    result = result && Convert.ToBoolean(oj);
                else
                    break;
            }
            return result;
        }

        public bool MonthlyRecurrenceFunction(string xml)
        {
            return MonthlyDOWRecurrenceFunction(xml);
        }

        public bool WeeklyRecurrenceFunction(string xml)
        {
            return MonthlyDOWRecurrenceFunction(xml);
        }

        public bool DailyRecurrenceFunction(string xml)
        {
            return MonthlyDOWRecurrenceFunction(xml);
        }
        //-------------------------------------------------------------------------------
        // Sub methods to handle sub options
        //-------------------------------------------------------------------------------
        public bool WhichWeekFunction(string xml)
        {
            XmlDocument xmldoc = new XmlDocument();
            xmldoc.LoadXml(xml);
            string WhichWeek = xmldoc["WhichWeek"].InnerText;
            double r = (nowDt.Day - (double)nowDt.DayOfWeek);//the month's day of last week sunday 
            int nowWeekInt = Convert.ToInt32(Math.Ceiling((r < 0 ? 0 : r) / 7)) + 1;//the index of the current week of the current month
            return WhichWeek == WhichWeekToName(nowWeekInt);
        }
        /// <summary>
        /// whether current datetime accord with the condition of every month's days
        /// </summary>
        /// <param name="xml"></param>
        /// <returns></returns>
        public bool DaysFunction(string xml)
        {
            XmlDocument xmldoc = new XmlDocument();
            xmldoc.LoadXml(xml);
            string day = xmldoc["Days"].InnerText;
            string[] days = day.Split(',');
            foreach (string dy in days)
            {
                if (dy.Length >= 3)
                {
                    string[] dys = dy.Split('-');
                    if (nowDt.Day >= Convert.ToInt32(dys[0]) && nowDt.Day <= Convert.ToInt32(dys[1]))
                        return true;
                }
                else
                {
                    if (nowDt.Day == Convert.ToInt32(dy))
                        return true;
                }
            }
            return false;
        }

        public bool WeeksIntervalFunction(string xml)
        {
            XmlDocument xmldoc = new XmlDocument();
            xmldoc.LoadXml(xml);
            int weeksInterval = Convert.ToInt32(xmldoc["WeeksInterval"].InnerText) + 1;
            double ts = (Math.Abs((nowDt - startDt).TotalDays) + Convert.ToInt32(startDt.DayOfWeek) - 7) / 7 % (weeksInterval);
            return ts < 1;
        }

        public bool DaysIntervalFunction(string xml)
        {
            XmlDocument xmldoc = new XmlDocument();
            xmldoc.LoadXml(xml);
            int daysInterval = Convert.ToInt32(xmldoc["DaysInterval"].InnerText);
            double ts = Math.Abs((nowDt - startDt).TotalDays) % daysInterval;
            return ts < 1;
        }

        public bool MonthsOfYearFunction(string xml)
        {
            XmlDocument xmldoc = new XmlDocument();
            xmldoc.LoadXml(xml);
            string monthStr = MonthToName(nowDt.Month);
            XmlElement nodeElement = xmldoc["MonthsOfYear"][monthStr];
            if (nodeElement != null && Convert.ToBoolean(nodeElement.InnerText))
                return true;
            else
                return false;
        }

        public bool DaysOfWeekFunction(string xml)
        {
            XmlDocument xmldoc = new XmlDocument();
            xmldoc.LoadXml(xml);
            string weekStr = WeekToName(Convert.ToInt32(nowDt.DayOfWeek));
            XmlElement nodeElement = xmldoc["DaysOfWeek"][weekStr];
            if (nodeElement != null && Convert.ToBoolean(nodeElement.InnerText))
                return true;
            else
                return false;
        }
        //-------------------------------------------------------------------------------
        private string WhichWeekToName(int weekOfMonth)
        {
            string nowWeekStr = "LastWeek";
            switch (weekOfMonth)
            {
                case 1:
                    nowWeekStr = "FirstWeek";
                    break;
                case 2:
                    nowWeekStr = "SecondWeek";
                    break;
                case 3:
                    nowWeekStr = "ThreeWeek";
                    break;
                case 4:
                    nowWeekStr = "FourtWeek";
                    break;
                default:
                    break;
            }
            return nowWeekStr;
        }

        private string MonthToName(int month)
        {
            string monthStr = "December";
            switch (month)
            {
                case 1:
                    monthStr = "January";
                    break;
                case 2:
                    monthStr = "February";
                    break;
                case 3:
                    monthStr = "March";
                    break;
                case 4:
                    monthStr = "April";
                    break;
                case 5:
                    monthStr = "May";
                    break;
                case 6:
                    monthStr = "June";
                    break;
                case 7:
                    monthStr = "July";
                    break;
                case 8:
                    monthStr = "August";
                    break;
                case 9:
                    monthStr = "September";
                    break;
                case 10:
                    monthStr = "October";
                    break;
                case 11:
                    monthStr = "November";
                    break;
                default:
                    break;

            }
            return monthStr;
        }

        private string WeekToName(int week)
        {
            string weekStr = "Sunday";
            switch (week)
            {
                case 1:
                    weekStr = "Monday";
                    break;
                case 2:
                    weekStr = "Tuesday";
                    break;
                case 3:
                    weekStr = "Tuesday";
                    break;
                case 4:
                    weekStr = "Thursday";
                    break;
                case 5:
                    weekStr = "Friday";
                    break;
                case 6:
                    weekStr = "Saturday";
                    break;
                default:
                    break;
            }
            return weekStr;
        }
    }
}